##############################################################################
##############################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# R program (does not use any BUGS model)
# A. salmon life cycle model
# Compare 10 alternative management strategies
# Chapter 12

##############################################################################
##############################################################################


rm(list = ls())
oldpath<-getwd()

library(tcltk)

source("routineSR2.R")
source("routineDecision.R")

rythme <- read.table(file="rythme.txt")

# Simulations (nombres d'annes et rptitions)
# ---------------------------------------------

Nyears <- 205
Nrep = 1000


# Migration rythm
# ---------------


def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "CumulatedReturns.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mar=c(5,5,3,3), bty="n")
size.text <- 1.3

plot(1:52,cumsum(c(rep(0,11),rythme[1:29,1],rep(0,12)))/sum(rythme[1:29,1]),
     xlim = c(1,52), ylab = "", xlab = "", xaxt="n", yaxt="n", main="", 
     col = "black", type = "l", lty = 1, lwd = 3)
lines(1:52,cumsum(c(rep(0,11),rythme[1:29,2],rep(0,12)))/sum(rythme[1:29,2]),
     ylab = "", xlab = "", xaxt="n", yaxt="n", main="", col = "black", type = "l", lty = 2, lwd = 3)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)
mtext(side=1, "Weeks", bty="n", line=3, cex=size.text)
mtext(side=2, "Cumulated return rates", bty="n", line=3, cex=size.text)

legend(	legend = c("1SW","2SW"),
 		col = c("black","black"), 	
 		lty = c(1,2), lwd = c(2,2), x = "topleft", cex = 1.5, bty ="n") 

par(def.par)

dev.off()



# Load the 10 management strategies
# -------------------------------------------

policies = read.table("policies.csv",head=T,sep=";")
policies$nom = as.character(policies$nom)
polvarie=1:length(policies$nom)



# Parameterization
# ---------------------------------------------------

# Spawners -> oeufs

pf1 <- 0.34 	
pf2 <- 0.68
fec1 <- 4635 	
fec2 <- 7965

# Eggs --> smolts
surf <- 209400 #m2
alpha1 <- -5.44 
alpha2 <- -6.71
beta <- 0.0163/surf
p <- 0.155
pvarie = c(0.01 , 0.0915, 0.1274 ,0.155 ,0.1895, 0.2414)
sigma <- 0.58

# pre-smolt -> smolt1
rho=0.89

# smolt -> spawners
pi11 <- 0.13
pi12 <- 0.03
pi21 <- 0.22
pi22 <- 0.04



# Characteristics

# Juveniles --> smolts
psi00 =(rho*pi11+(1-rho)*pi21)*(pf1*fec1)+(rho*pi12+(1-rho)*pi22)*(pf2*fec2)

# equivalent ricker oeuf-> juvenile
alphaprime =(sigma^2)/2+log((1-p)*exp(alpha1)+p*exp(alpha2))
betaprime = beta

# optimal harvest rate
htest = seq(0,0.9,by=0.0001)
alphatest = htest-log(1-htest)
hopt = htest[which.min((log(psi00)+alphaprime-alphatest)^2)]
Wopt = hopt/betaprime
Nopt = SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*(1-hopt)
Wopt/psi00
Nh =((log(1-hopt)+log(psi00)+alphaprime)/(betaprime*(1-hopt)*psi00))
SR2(Wopt,alphaprime,alphaprime,betaprime,0.5,0.00001,1)*psi00*(1-hopt)-Wopt
R1opt = round(Nopt*(rho*pi11+(1-rho)*pi21)         )
R2opt = round((rho*pi12+(1-rho)*pi22)*Nopt     )


# Simulations
# ------------------------------------------------------------------------------------


findeserie<-matrix(NA, ncol=6, nr=Nrep)
period=1:(Nyears-2)
periodmean=(Nyears-200+20):(Nyears-100+20)

pourcent=1 		# initlaisation = 1% ref. state

pextinct1=matrix(0,length(pvarie), length(polvarie))
pextinct2=pextinct1
Alevinsstatio=pextinct1
Oeufsstatio=pextinct1
Alevinsmean=pextinct1
Alevinsvar=pextinct1
Alevinsq10=pextinct1
Oeufsmean=pextinct1
Oeufsq10=pextinct1
HM2mean=pextinct1
HM2var=pextinct1
HM2q10=pextinct1
HM1mean=pextinct1
HM1var=pextinct1
HM1q10=pextinct1
D1mean=pextinct1
D1var=pextinct1
D1q10=pextinct1
D2mean=pextinct1
D2var=pextinct1
D2q10=pextinct1

Benef001 = pextinct1
Benef003 = pextinct1
Benef007 = pextinct1

discount001 <- matrix(exp(-0.01*(0:length(periodmean)-1)),nr=length(periodmean), nc=1)
discount003 <- matrix(exp(-0.03*(0:length(periodmean)-1)),nr=length(periodmean), nc=1)
discount007 <- matrix(exp(-0.07*(0:length(periodmean)-1)),nr=length(periodmean), nc=1)



ptm <- proc.time()

pb <- tkProgressBar(title = "R progression", width = 1000,
                    min = min(polvarie), max = max(polvarie), initial = min(polvarie))



# Loop on value of p
# ---------------------------------------------

indicep=0

for (p in pvarie) {

indicep=indicep+1
print(pvarie[indicep])
print(ptm-proc.time())
ptm<-proc.time()

initoeuf<-Wopt*pourcent
initalevins=ceiling(Nh*pourcent)
initsmolt1=ceiling(rho*Nh*pourcent)
initsmolt2=initalevins-initsmolt1
inithm11=ceiling(pi11*initsmolt1)
inithm12=ceiling(pi12*initsmolt1)
inithm21=ceiling(pi21*initsmolt2)
inithm22=ceiling(pi22*initsmolt2)


 
# Loop on strategies
# ---------------------------------------------

indicepol = 0

for (pol in polvarie) {

setTkProgressBar(pb,pol,label = paste("Running", length(polvarie),"strategies for val p = ",which(pvarie==p)," on ", length(pvarie)))

indicepol=indicepol+1
inithm1=ceiling((inithm11+inithm21))
#*(1-h))
inithm2=ceiling((inithm12+inithm22))
#*(1-h))
initoeuf<-ceiling((inithm1*fec1*pf1+inithm2*fec2*pf2))

Oeufs<-matrix(initoeuf,ncol=Nyears,nr=Nrep)
Alevins<-matrix(initalevins,ncol=Nyears,nr=Nrep)
Smolt1<-matrix(initsmolt1,ncol=Nyears,nr=Nrep)
Smolt2<-matrix(initsmolt2,ncol=Nyears,nr=Nrep)
SW11<-matrix(inithm11,ncol=Nyears,nr=Nrep)
SW12<-matrix(inithm12,ncol=Nyears,nr=Nrep)
SW21<-matrix(inithm21,ncol=Nyears,nr=Nrep)
SW22<-matrix(inithm22,ncol=Nyears,nr=Nrep)
HM2<-matrix(inithm2,ncol=Nyears,nr=Nrep)
HM1<-matrix(inithm1,ncol=Nyears,nr=Nrep)
D1<-matrix(0,ncol=Nyears,nr=Nrep)
D2<-matrix(0,ncol=Nyears,nr=Nrep)
#fin d Initialisation
Oeufs[,(Nyears-5):Nyears]<-findeserie
Alevins[,(Nyears-5):Nyears]<-findeserie
Smolt1[,(Nyears-5):Nyears]<-findeserie
Smolt2[,(Nyears-5):Nyears]<-findeserie
SW11[,(Nyears-5):Nyears]<-findeserie
SW12[,(Nyears-5):Nyears]<-findeserie
SW21[,(Nyears-5):Nyears]<-findeserie
SW22[,(Nyears-5):Nyears]<-findeserie
HM2[,(Nyears-5):Nyears]<-findeserie
HM1[,(Nyears-5):Nyears]<-findeserie

# State dynamics t --> t+1 / t+2
# ---------------------------------------------

for (t in 1:(Nyears-2)){

HM1[,t]<-floor((SW21[,t]+SW11[,t]))
HM2[,t]<-floor((SW22[,t]+SW12[,t]))
    deci=Decisionmvt(N1=HM1[,t],p1=0.05,debut1=policies$deb1[pol]-11,fin1=policies$fin1[pol]-11,
    N2=HM2[,t],p2=0.051,debut2=policies$deb2[pol]-11,fin2=policies$fin2[pol]-11,
    TAC=policies$TAC[pol],TAC2=policies$TAC2[pol],Sopt=policies$Sopt[pol],regle=policies$nom[pol],rythme)
D1[,t]=deci[,1]
D2[,t]=deci[,2]
Oeufs[,t]<-floor(((HM1[,t]-D1[,t])*pf1*fec1+(HM2[,t]-D2[,t])*pf2*fec2))
#projection
#SR
Alevins[,t+1]<- SR2(Oeufs[,t],alpha1,alpha2,beta, p,sigma,Nrep)	
#p est ici dans la boucle sur indicep p=pvarie[indicep]
#Smolts
Smolt1[,t+1]<-rbinom(Nrep,Alevins[,t],rho)
Smolt2[,t+2]<-(Alevins[,t])-Smolt1[,t+1]
#Retours ulterieurs
SW12[,t+2]<-rbinom(Nrep,Smolt1[,t],pi12)
SW11[,t+1]<-rbinom(Nrep,Smolt1[,t],pi11)
SW22[,t+2]<-rbinom(Nrep,Smolt2[,t],pi22)
SW21[,t+1]<-rbinom(Nrep,Smolt2[,t],pi21)

} # end loop on t



zeros=which(Alevins[,Nyears-2]==0)
Tsurv=(Nrep-length(zeros))*length(period)-min(period-1)*length(zeros)
for (i in zeros){
Tsurv=Tsurv+min(which(Alevins[i,period]==0))

} # end loop on i

pextinct1[indicep,indicepol]=length(zeros)/Nrep

# proba extinct in 200 years

pextinct2[indicep,indicepol]=(Nrep*length(period)-Tsurv)/(Nrep*length(period))
Alevinsstatio[indicep,indicepol]=mean(Alevins[-zeros,periodmean])
Oeufsstatio[indicep,indicepol]=mean(Oeufs[-zeros,periodmean])
Alevinsmean[indicep,indicepol]=mean(Alevins[,periodmean])
Alevinsvar[indicep,indicepol]=var(as.vector(Alevins[,periodmean]))
Oeufsmean[indicep,indicepol]=mean(Oeufs[,periodmean])
Alevinsq10[indicep,indicepol]=mean(apply(Alevins[,periodmean],2,quantile,probs=c(0.1)))
Oeufsmean[indicep,indicepol]=mean(Oeufs[,periodmean])
Oeufsq10[indicep,indicepol]=mean(apply(Alevins[,periodmean],2,quantile,probs=c(0.1)))

HM2mean[indicep,indicepol]=mean(HM2[,periodmean])
HM2var[indicep,indicepol]=var(as.vector(HM2[,periodmean]))
HM2q10[indicep,indicepol]=mean(apply(HM2[,periodmean],2,quantile,probs=c(0.1)))
HM1mean[indicep,indicepol]=mean(HM1[,periodmean])
HM1var[indicep,indicepol]=var(as.vector(HM1[,periodmean]))
HM1q10[indicep,indicepol]=mean(apply(HM1[,periodmean],2,quantile,probs=c(0.1)))

D2mean[indicep,indicepol]=mean(D2[,periodmean])
D2var[indicep,indicepol]=var(as.vector(D2[,periodmean]))
D2q10[indicep,indicepol]=mean(apply(D2[,periodmean],2,quantile,probs=c(0.1)))
D1mean[indicep,indicepol]=mean(D1[,periodmean])
D1var[indicep,indicepol]=var(as.vector(D1[,periodmean]))
D1q10[indicep,indicepol]=mean(apply(D1[,periodmean],2,quantile,probs=c(0.1)))

Benef001[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=2,p3=0,R1=1,R2=1)%*%discount001)
Benef003[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=2,p3=0,R1=1,R2=1)%*%discount003)
Benef007[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=2,p3=0,R1=1,R2=1)%*%discount007)

#Benef001[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=10,p3=0,R1=1,R2=1)%*%discount001)
#Benef003[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=10,p3=0,R1=1,R2=1)%*%discount003)
#Benef007[indicep,indicepol]=mean(Benef(D1[,periodmean],D2[,periodmean],p1=1,p2=10,p3=0,R1=1,R2=1)%*%discount007)

} # end loop on strategies

} # end loop on p




# ---------------------------------------------
#                    FIGURES
# ---------------------------------------------


polencourse=c(1:10)



# Escapement 1SW and 2SW and variance (coeficient of variation)
# -------------------------------------------------
 
mean1 <- HM1mean[3,polencourse]-D1mean[3,polencourse]
var1 <- HM1var[3,polencourse]
CV1 <- (var1^0.5)/mean1

mean2 <- HM2mean[3,polencourse]-D2mean[3,polencourse]
var2 <- HM2var[3,polencourse]
CV2 <- (var2^0.5)/mean2


# windows()

res <- 5
name_figure <- "Returns.png"
png(filename = name_figure, height = 500*res, width = 750*res, res=72*res)

par(mfrow=c(1,2), bty="n")
size.text <- 1.3

plot(mean1,CV1,xlab="Mean escapement",
     xaxt="n", yaxt="n",
	ylab="Coef. of variation of returns",main="a) Grilses", 
	xlim=c(0,2600),ylim=c(0,8), pch=19, cex.lab=size.text)
pos = c(4,1,1,1,4,4,4,4,1,3)
text(mean1,CV1, polencourse, cex=size.text, pos = pos)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)


plot(mean2,CV2,xlab="Mean escapement",
	xaxt="n", yaxt="n",     
	ylab="Coef. of variation of returns", main="b) Spring salmons", 
	xlim=c(0,600), ylim=c(0,14), pch=19, cex.lab=size.text)
pos = c(4,1,1,4,4,4,4,3,1,4)
text(mean2, CV2, polencourse, cex=size.text, pos = pos)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)

dev.off()




# Exploitation 2SW and all sea-age classes
# ---------------------------------------

# windows()

res <- 5
name_figure <- "Exploitation.png"
png(filename = name_figure, height = 500*res, width = 750*res, res=72*res)

par(mfrow = c(1,2), bty="n")
size.text <- 1.3

# Spring salmons

returns2 <- HM2mean[3,polencourse]
catches2 <- D2mean[3,polencourse]

plot( returns2, catches2, xlab="Mean returns",
	xaxt="n", yaxt="n", 
	ylab="Mean catches",main="a) Spring salmons", 
	xlim=c(0,600),ylim=c(0,150), pch=19, cex.lab=size.text)
pos = c(4,4,4,3,4,4,4,4,4,1)
text(returns2,catches2,polencourse, cex=size.text, pos = pos)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)

# Total 

returnsT <- HM2mean[3,polencourse] + HM1mean[3,polencourse]
catchesT <- D2mean[3,polencourse] + D1mean[3,polencourse]

plot(returnsT,catchesT,xlab="Mean returns",ylab="Mean catches",main="b) All sea-age classes",
     xaxt="n", yaxt="n", 
	xlim=c(0,3500),ylim=c(0,300), , pch=19, cex.lab=size.text)
pos = c(4,4,4,3,4,4,4,4,4,4)
text(returnsT, catchesT, polencourse, cex=size.text, pos = pos)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)

dev.off()



# Benefits
# ----------------------------------------

# windows()

res <- 6
name_figure <- "Benefits003.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

image(Benef003[,polencourse],x=pvarie,y=1:length(polencourse),xlab="uncertainty on p",
      ylab="policies",col=terrain.colors(10))
text(-0.01,1:length(polencourse),polencourse, cex=size.text)
for (i in 1:6){text(pvarie[i],1:length(polencourse),(format(0.001*Benef003[i,polencourse],dig=1)))}

dev.off()



# windows()

res <- 6
name_figure <- "Benefits007.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

image(Benef007[,polencourse],x=pvarie,y=1:length(polencourse),xlab="uncertainty on p", 
      ylab="policies",col=terrain.colors(10))
text(-0.01,1:length(polencourse),polencourse, cex=size.text)
for (i in 1:6){text(pvarie[i],polvarie,(9-rank(Benef007[i,polencourse])))}

dev.off()




# P extinction (for the 10 strategies) as a function of p
# -------------------------------------------------------

size.text <- 1.3

windows()

res <- 6
name_figure <- "ProbExtinct_vs_cata.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

image(pextinct1<1, x=pvarie,y=1:length(policies$nom),xlab="", ylab="", yaxt="n", xaxt="n")
#,col=terrain.colors(10))

text(-0.02,polvarie,polvarie, cex=1.5)
# text(0.03,polvarie,policies$englishname)

axis(side = 1, tick = T, cex.axis=1, las=1)
# axis(side = 2, tick = T, cex.axis=1)
mtext(side=1, "Value of p", bty="n", line=3, cex=size.text)
mtext(side=2, "Policies", bty="n", line=1, cex=size.text)

for (i in 2:6){ text(pvarie[i],polvarie,pextinct1[i,polvarie],dig=2) }

dev.off()



# windows()

res <- 6
name_figure <- "ProbExtinct_vs_strategies.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

plot(polvarie,pextinct1[1,],typ='b',xlab="Strategies", ylab="Prob. of extinction", pch=1, col=1)
for (i in 2:length(pvarie)){
lines(polvarie,pextinct1[i,],col=i, pch=i, typ='b')
}

legend("topleft", legend=pvarie, col=1:length(pvarie), text.col=1:length(pvarie),
       pch=1:length(pvarie), title="Init.stock")

dev.off()


